(in-package #:asdf-status)

(defun generate-summaries-by-author (systems)
  (let ((authors (all-authors systems)))
    ;; one index of authors
    (generate-author-index systems authors)
    
    ;; one summary page for each author
    (iterate-elements
     authors
     (lambda (author)
       (generate-status-pages 
        :systems systems
        :filter (make-author-filter author)
        :output (make-author-index-name author)
        :make-detail-pages? nil)))))

;;; ---------------------------------------------------------------------------

(defun make-author-filter (author)
  (lambda (x) (string-equal (getf x :author) author)))

;;; ---------------------------------------------------------------------------

(defun make-author-index-name (author)
  (if (null author)
    (path-of-summary-page "index-unknown-authors")
    (path-of-summary-page (concatenate 'string "index-" author))))

;;; ---------------------------------------------------------------------------

(defun generate-author-index (systems authors)
  (let ((all-implementations 
         (all-implementations-for-matrix (systems-matrix systems))))
    (html-page ((path-of-summary-page "index-authors"))
      (html
       (:HEAD
        (:TITLE "ASDF System Status - Author Index")
        ((:LINK :REL "stylesheet" :TYPE "text/css" :HREF "system-colors.css" :TITLE "combined"))
        ((:LINK :REL "stylesheet" :TYPE "text/css" :HREF "system-shared.css" :TITLE "combined"))
        ((:LINK :REL "stylesheet" :TYPE "text/css" :HREF "system-summary.css" :TITLE "combined"))
        ((:META :HTTP-EQUIV "Content-Type" :CONTENT "text/html; charset=ISO-8859-1")))
       (:BODY
        ((:DIV :CLASS "header")
         ((:SPAN :CLASS "logo") ((:IMG :SRC "lisp-flag-with-hand.gif")))
         (:H2 "ASDF System Status - Author Index")
         (:H4 "Generated on "
              (lml-princ (metatilities:format-date "%A, %e %B %Y" (get-universal-time)))))
        
        (:P "Brought to you by " 
            ((:A :HREF "http://common-lisp.net/project/ait/")
             "ASDF-Install-Tester") ". Remember, when ASDF is happy, you're happy too.")
        
        (:P "Back to " ((:a :href "index.html") "main index"))
        
        (html 
         ((:table :class "table")
          (:tr
           (:td
            ((:table :class "matrix") 
             (:tr
              (:td)
              (loop for nil in all-implementations 
                    for code in *implementation-codes* do
                    (html ((:th :colspan 2 :class "implementation-code") (lml-princ code)))))
             (iterate-elements
              (sort (copy-list authors) #'string-lessp)
              (lambda (author)
                (let* ((filter (make-author-filter author))
                       ;; (matrix (systems-matrix systems :filter filter))
                       (summary-counts (system-summary-counts systems filter))
                       (total-good 0)
                       (total-bad 0))
                  (flet ((sub-total (implementation value)
                           (or (second (assoc (list implementation value) summary-counts
                                              :test #'equal))
                               0)))
                    (html 
                     (:tr
                      ((:th :class "author-name")
                       ((:a :href (namestring (pathname-name+type (make-author-index-name author))))
                            (lml2::princ-safe-http (if (null author) "Unknown" author))))
                      
                      (loop for implementation in all-implementations do
                            (let ((good (sub-total implementation t))
                                  (bad (sub-total implementation nil)))
                              (incf total-good good)
                              (incf total-bad bad)
                              (if (zerop good) 
                                (html (:td))
                                (html ((:td :class "asdf-success")
                                       (lml-princ (format nil "~:D" good)))))
                              (if (zerop bad)
                                (html (:td))
                                (html ((:td :class "asdf-general-error")
                                       (lml-princ (format nil "~:D" bad)))))))))))))))
           
           ((:td :class "hide-me") "+++")
           
           (:td 
            (generate-keys all-implementations 
                           :links nil
                           :simple-only? t)))
          
          (:tr ((:td :colspan 2) (generate-footer-links))))))))))