(in-package #:asdf-status)

(defparameter *mcl-systems*
  (remove-if
   #'atom
   (mapcar 
    (lambda (f)
      (with-open-file (in f)
        (handler-case
          (aprog1 
            (read in nil nil)
            (when (atom it)
              (warn "Parse error: ~A" f)))
          (error () (warn "Read error: ~A" f)))))
    (directory (make-pathname
                :name :wild
                :type "ait"
                :directory `(,@(pathname-directory *input-directory*)
                             "openmcl"
                             :wild-inferiors)
                :defaults *input-directory*)))))

(defparameter *dependent-systems*
  (remove-if 
   (lambda (system-info)
     (null (getf system-info :depends-on)))
   *mcl-systems*))

(defmethod coerce-system-name ((name string))
  (string-downcase name))

(defmethod coerce-system-name ((name symbol))
  (coerce-system-name (symbol-name name)))

                               
(defparameter *systems-graph*
  (let ((g (cl-graph:make-graph 'cl-graph:graph-container :vertex-test #'equal)))
    (iterate-elements 
     *dependent-systems*
     (lambda (system-info)
       (let ((system (coerce-system-name (getf system-info :install))))
         (cl-graph:add-vertex g system)
         (iterate-elements
          (getf system-info :depends-on)
          (lambda (other)
            (cl-graph:add-edge-between-vertexes g (coerce-system-name other) system
                                                :edge-type :directed))))))
    
    g))

(cl-graph:graph->dot 
 *systems-graph* "user-home:docs;foo.dot"
 :vertex-labeler
 (lambda (v s) (princ (element v) s))
 :vertex-formatter
 (lambda (v s) (format s "URL=\"http://www.cliki.net/~(~A~)\"" (element v))))